# Generated by roxygen2: do not edit by hand

S3method("+",tf_ggplot)
S3method(autolayer,tf)
S3method(autoplot,tf)
S3method(ggplot_build,tf_ggplot)
S3method(is.finite,tf)
S3method(print,tf_ggplot)
S3method(scale_type,tf)
S3method(tf_evaluate,data.frame)
S3method(tf_unnest,data.frame)
S3method(tf_unnest,tf)
export(GeomCapellini)
export(GeomErrorband)
export(GeomFboxplot)
export(GeomMeatballs)
export(GeomSpaghetti)
export(StatCapellini)
export(StatErrorband)
export(StatFboxplot)
export(StatTf)
export(autolayer)
export(autoplot)
export(geom_capelini)
export(geom_capellini)
export(geom_cappelini)
export(geom_cappellini)
export(geom_errorband)
export(geom_fboxplot)
export(geom_meatballs)
export(geom_spaghetti)
export(gglasagna)
export(is_tf_ggplot)
export(parse_tf_aesthetics)
export(stat_capellini)
export(stat_errorband)
export(stat_fboxplot)
export(stat_tf)
export(tf_gather)
export(tf_ggplot)
export(tf_nest)
export(tf_spread)
export(tf_unnest)
import(dplyr)
import(ggplot2)
import(rlang)
import(tf)
importFrom(GGally,rescale01)
importFrom(GGally,rescale11)
importFrom(ggplot2,autolayer)
importFrom(ggplot2,autoplot)
importFrom(grid,gList)
importFrom(grid,gpar)
importFrom(grid,grobTree)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(pillar,obj_sum)
importFrom(pillar,pillar_shaft)
importFrom(pillar,type_sum)
importFrom(purrr,map)
importFrom(purrr,map_lgl)
importFrom(purrr,pmap)
importFrom(tibble,tibble)
importFrom(tidyr,unnest)
importFrom(tidyselect,eval_select)
importFrom(tidyselect,vars_pull)
