#' @description
#' `tidyfun` makes data wrangling and exploratory analysis for functional data
#' in `R` easier.\cr\cr
#' The `tf` package defines the core functional data classes and methods.
#' `tidyfun` builds on those objects with:
#'
#'   - data-frame workflows for functional data: [tf_gather()] etc.
#'   - graphics helpers for `tf` objects and tidy functional data frames: [geom_spaghetti()], [gglasagna()]
#'   - documentation and examples that show how to use `tf` objects in tidy workflows
#' \cr\cr
#' \cr
#' Start with `vignette(package = "tidyfun")` for the article index.
#'
#' @keywords internal
#' @import rlang
"_PACKAGE"
