% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tf-ggplot.R
\name{parse_tf_aesthetics}
\alias{parse_tf_aesthetics}
\title{Parse aesthetic mappings to separate tf and regular aesthetics}
\usage{
parse_tf_aesthetics(mapping, data = NULL)
}
\arguments{
\item{mapping}{An aesthetic mapping created with \code{\link[ggplot2:aes]{ggplot2::aes()}}.}

\item{data}{Data frame to evaluate expressions against.}
}
\value{
A list with components \code{tf_aes}, \code{scalar_tf_aes}, and \code{regular_aes}.
}
\description{
Parse aesthetic mappings to separate tf and regular aesthetics
}
\examples{
parse_tf_aesthetics(ggplot2::aes(tf = f, color = group))
parse_tf_aesthetics(ggplot2::aes(x = x, y = y))
}
