% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-spaghetti.R
\docType{data}
\name{ggspaghetti}
\alias{ggspaghetti}
\alias{StatTf}
\alias{stat_tf}
\alias{geom_spaghetti}
\alias{GeomSpaghetti}
\alias{geom_meatballs}
\alias{GeomMeatballs}
\title{Spaghetti plots for \code{tf} objects}
\usage{
stat_tf(
  mapping = NULL,
  data = NULL,
  geom = "spaghetti",
  position = "identity",
  na.rm = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  arg = NULL,
  ...
)

geom_spaghetti(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  arg = NULL,
  ...
)

geom_meatballs(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  arg = NULL,
  spaghetti = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{remove NAs? defaults to \code{TRUE}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:annotation_borders]{annotation_borders()}}.}

\item{arg}{where to evaluate the functions in \code{y}; defaults to the object's
default evaluation grid.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}

\item{spaghetti}{plot noodles along with meatballs? defaults to TRUE.}
}
\value{
A \code{\link[ggplot2:layer]{ggplot2::layer()}} object for use in a ggplot.
}
\description{
Plots a line for each entry of a \code{tf}-object.
\code{geom_spaghetti} draws a line through each function, and \code{geom_meatballs}
adds points for the evaluated grid values.
}
\details{
"Flipped" aesthetics are not implemented for these geoms.
}
\section{\code{y} aesthetic}{

Mandatory. Used to designate a column of class \code{tf} to be visualized.
}

\examples{
set.seed(1221)
data <- data.frame(col = sample(gl(5, 2)))
data$f <- tf_rgp(10)
data$fi <- tf_jiggle(data$f)
data$fb <- tfb(data$f)
library(ggplot2)
ggplot(data, aes(y = f, color = tf_depth(f))) +
  geom_spaghetti()
ggplot(data, aes(y = fi, shape = col, color = col)) +
  geom_meatballs()
ggplot(data, aes(y = fi)) +
  geom_meatballs(spaghetti = FALSE) +
  facet_wrap(~col)
}
\seealso{
\code{\link[=geom_cappelini]{geom_cappelini()}} for glyph plots, \code{\link[=gglasagna]{gglasagna()}} for heatmaps.

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}

Other tidyfun visualization: 
\code{\link{autoplot.tf}()},
\code{\link{ggcapellini}},
\code{\link{gglasagna}()}
}
\concept{tidyfun visualization}
\keyword{datasets}
